@@ -10,7 +10,7 @@ use mongodb::options::{
 };
 use mongodb::results::{DeleteResult, InsertManyResult, UpdateResult};
 
-use super::cursor::Cursor;
+use super::cursor::{Cursor, TypedCursor};
 use crate::collection::Collection;
 use crate::filter::{AsFilter, Filter};
 use crate::query;
@@ -218,7 +218,7 @@ impl Client {
     /// # Errors
     ///
     /// This method fails if the mongodb encountered an error.
-    pub fn delete<C, F>(&self, filter: Option<F>) -> crate::Result<i64>
+    pub fn delete<C, F>(&self, filter: Option<F>) -> crate::Result<u64>
     where
         C: AsFilter<F> + Collection,
         F: Filter,
@@ -227,7 +227,7 @@ impl Client {
         if let Some(filter) = filter {
             delete = delete.filter(filter)?;
         }
-        delete.blocking(&self)
+        delete.blocking(self)
     }
 
     /// Convenience method to delete one document from a collection using a given filter.
@@ -243,7 +243,7 @@ impl Client {
         let deleted = query::Delete::<C>::new()
             .many(false)
             .filter::<F>(filter)?
-            .blocking(&self)?;
+            .blocking(self)?;
         Ok(deleted > 0)
     }
 
@@ -255,7 +255,7 @@ impl Client {
     /// # Errors
     ///
     /// This method fails if the mongodb encountered an error.
-    pub fn find<C, F>(&self, filter: Option<F>) -> crate::Result<Cursor>
+    pub fn find<C, F>(&self, filter: Option<F>) -> crate::Result<TypedCursor<C>>
     where
         C: AsFilter<F> + Collection,
         F: Filter,
@@ -264,7 +264,7 @@ impl Client {
         if let Some(filter) = filter {
             find = find.filter(filter)?;
         }
-        find.blocking(&self)
+        find.blocking(self)
     }
 
     /// Convenience method to find a document in a collection using a given filter.
@@ -275,18 +275,16 @@ impl Client {
     /// # Errors
     ///
     /// This method fails if the mongodb encountered an error, or if the found document is invalid.
-    pub fn find_one<C, F>(&self, filter: F) -> crate::Result<Option<C>>
+    pub fn find_one<C, F>(&self, filter: F) -> crate::Result<Option<(ObjectId, C)>>
     where
         C: AsFilter<F> + Collection,
         F: Filter,
     {
         // NOTE: We don't wanna make another builder so we just eat the cost of getting a cursor...
         let find: query::Find<C> = query::Find::new();
-        let mut cursor = find.filter(filter)?.blocking(&self)?;
+        let mut cursor = find.filter(filter)?.blocking(self)?;
         if let Some(res) = cursor.next() {
-            let doc = res.map_err(crate::error::mongodb)?;
-            let document: C = C::from_document(doc).map_err(crate::error::bson)?;
-            return Ok(Some(document));
+            return Ok(Some(res?));
         }
         Ok(None)
     }
@@ -300,7 +298,7 @@ impl Client {
     where
         C: Collection,
     {
-        let result = query::Insert::new().blocking(&self, documents)?;
+        let result = query::Insert::new().blocking(self, documents)?;
         Ok(result
             .into_iter()
             .filter_map(|(k, v)| match v {
@@ -320,11 +318,9 @@ impl Client {
         C: Collection,
     {
         // NOTE: We don't wanna make another builder so we just eat the cost of allocating a vec...
-        let result = query::Insert::new().blocking(&self, vec![document])?;
-        if let Some((_, v)) = result.into_iter().next() {
-            if let bson::Bson::ObjectId(id) = v {
-                return Ok(id);
-            }
+        let result = query::Insert::new().blocking(self, vec![document])?;
+        if let Some((_, bson::Bson::ObjectId(id))) = result.into_iter().next() {
+            return Ok(id);
         }
         Err(crate::error::mongodb(
             "failed to insert document into mongo",
@@ -343,7 +339,7 @@ impl Client {
     {
         query::Replace::new()
             .filter::<F>(filter)?
-            .blocking(&self, document)
+            .blocking(self, document)
     }
 
     /// Convenience method to update documents in a collection.
@@ -351,7 +347,7 @@ impl Client {
     /// # Errors
     ///
     /// This method fails if the mongodb encountered an error.
-    pub fn update<C, F, U>(&self, filter: F, updates: Updates<U>) -> crate::Result<i64>
+    pub fn update<C, F, U>(&self, filter: F, updates: Updates<U>) -> crate::Result<u64>
     where
         C: AsFilter<F> + AsUpdate<U> + Collection,
         F: Filter,
@@ -359,7 +355,7 @@ impl Client {
     {
         let updated = query::Update::<C>::new()
             .filter::<F>(filter)?
-            .blocking::<U>(&self, updates)?;
+            .blocking::<U>(self, updates)?;
         Ok(updated)
     }
 
@@ -377,7 +373,7 @@ impl Client {
         let updated = query::Update::<C>::new()
             .many(false)
             .filter::<F>(filter)?
-            .blocking::<U>(&self, updates)?;
+            .blocking::<U>(self, updates)?;
         if updated > 0 {
             return Ok(true);
         }
@@ -389,7 +385,7 @@ impl Client {
     /// # Errors
     ///
     /// This method fails if the mongodb encountered an error.
-    pub fn upsert<C, F, U>(&self, filter: F, updates: Updates<U>) -> crate::Result<i64>
+    pub fn upsert<C, F, U>(&self, filter: F, updates: Updates<U>) -> crate::Result<u64>
     where
         C: AsFilter<F> + AsUpdate<U> + Collection,
         F: Filter,
@@ -398,7 +394,7 @@ impl Client {
         let updated = query::Update::<C>::new()
             .upsert(true)
             .filter::<F>(filter)?
-            .blocking::<U>(&self, updates)?;
+            .blocking::<U>(self, updates)?;
         Ok(updated)
     }
 
@@ -417,7 +413,7 @@ impl Client {
             .many(false)
             .upsert(true)
             .filter::<F>(filter)?
-            .blocking::<U>(&self, updates)?;
+            .blocking::<U>(self, updates)?;
         if updated > 0 {
             return Ok(true);
         }
@@ -476,12 +472,12 @@ impl ClientInner {
                         let resp = match req {
                             Request::Delete(many, collection, filter, options) => if many {
                                 database
-                                    .collection(collection)
+                                    .collection::<Document>(collection)
                                     .delete_many(filter, options)
                                     .await
                             } else {
                                 database
-                                    .collection(collection)
+                                    .collection::<Document>(collection)
                                     .delete_one(filter, options)
                                     .await
                             }
@@ -508,12 +504,12 @@ impl ClientInner {
                             Request::Update(many, collection, filter, updates, options) => {
                                 if many {
                                     database
-                                        .collection(collection)
+                                        .collection::<Document>(collection)
                                         .update_many(filter, updates, options)
                                         .await
                                 } else {
                                     database
-                                        .collection(collection)
+                                        .collection::<Document>(collection)
                                         .update_one(filter, updates, options)
                                         .await
                                 }
