@@ -27,8 +27,7 @@
 //!
 //! let mut cursor = client.find::<User, _>(None).unwrap();
 //! while let Some(res) = cursor.next() {
-//!     if let Ok(doc) = res {
-//!         let user: User = User::from_document(doc).unwrap();
+//!     if let Ok(user) = res {
 //!         println!("{:?}", user);
 //!     }
 //! }
@@ -39,4 +38,4 @@ mod cursor;
 
 pub use self::client::{Client, ClientBuilder};
 pub(crate) use self::client::{Request, Response};
-pub use self::cursor::Cursor;
+pub use self::cursor::{Cursor, TypedCursor};
